using System;
using System.Collections;
using System.Configuration;
using System.Globalization;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.SessionState;
using System.Xml.XPath;
using MailBee.Html;
using MailBee.ImapMail;
using MailBee.Mime;
using Microsoft.Win32;

namespace WebMailLite
{
	//*******************************************************************
	//By ZaKon: validation class
	//*******************************************************************
	public class Validation
	{
		private static string _errorMessage = null;
		private static string _corrected = null;
		private static string _testData = null;
		private static object _additionalInfo = null;
			
		public static string ErrorMessage
		{
			get
			{
				return _errorMessage;
			}
		}
		public static string Corrected
		{
			get
			{
				return _corrected;
			}
		}
		//Enum			
		public enum ValidationTask
		{
			/// <summary>
			/// Input: Not empty email string, like account@domen. Output: Trimmed string.
			/// </summary>
			Email,
			/// <summary>
			/// Input: Not empty login string, without special chars. Output: Trimmed string. Additional info: advanced login flag as string: "0" || "1"
			/// </summary>
			Login,
			/// <summary>
			/// Input: Not empty string. Output: Input.
			/// </summary>
			Password,
			/// <summary>
			/// Input: Incoming email server name, not empty. Output: Trimmed string.
			/// </summary>
			INServer,
			/// <summary>
			/// Input: Positive integer number for incoming port, not empty. Output: Trimmed string.
			/// </summary>
			INPort,
			/// <summary>
			/// Input: Outcoming email server name, not empty. Output: Trimmed string.
			/// </summary>
			OUTServer,
			/// <summary>
			/// Input: Positive integer number for outcoming port, not empty. Output: Trimmed string.
			/// </summary>
			OUTPort,
			/// <summary>
			/// Input: Messages per page value as positive number. Output: Trimmed string.
			/// </summary>
			MPP,
			/// <summary>
			/// Input: (???) Advanced, not empty. Output: Trimmed string.
			/// </summary>
			Advanced,
			/// <summary>
			/// Input: SMTP Server login. Output: Trimmed string.
			/// </summary>
			SMTPLogin,
			/// <summary>
			/// Input: Keep messages on server for days, positive number, not empty. Output: Trimmed string.
			/// </summary>
			KeepMessages,
			/// <summary>
			/// Input: Filter substring, not empty. Output: Trimmed string.
			/// </summary>
			Substring,
			/// <summary>
			/// Input: Valid folder name, not empty, excluding CON, AUX, COM1, COM2, COM3, COM4, LPT1, LPT2, LPT3, PRN, NUL. Output: Trimmed string.
			/// </summary>
			FolderName
		}
		//Ctor
		public Validation()
		{
		}
		//Public methods
		public static bool CheckIt(ValidationTask checkFor, string testData)
		{
			return CheckIt(checkFor, testData, null);
		}
		public static bool CheckIt(ValidationTask checkFor, string testData, object additionalInfo)
		{
			if (testData==null)
			{
				_errorMessage = "Null!";
				return false;
			}
			_testData = testData;
			if (additionalInfo!=null)
				{
					_additionalInfo = additionalInfo;
				}
			switch(checkFor)
			{
				case ValidationTask.Email:
					return emailCheck();
				case ValidationTask.Login:
					return loginCheck();
				case ValidationTask.Password:
					return passwordCheck();
				case ValidationTask.INServer:
					return inServerCheck();
				case ValidationTask.INPort:
					return inPortCheck();
				case ValidationTask.OUTServer:
					return outServerCheck();
				case ValidationTask.OUTPort:
					return outPortCheck();
				case ValidationTask.MPP:
					return mppCheck();
				case ValidationTask.Advanced:
					return advancedCheck();
				case ValidationTask.SMTPLogin:
					return smtpLogin();
				case ValidationTask.KeepMessages:
					return keepMessagesCheck();
				case ValidationTask.Substring:
					return substringCheck();
				case ValidationTask.FolderName:
					return folderNameCheck();
				default:
					return true;
			}
		}
		//Checks
		/// <summary>
		/// Check for valid email. Additional info required:
		/// </summary>
		/// <returns>True, if valid. False, if invalid.</returns>
		private static bool emailCheck()
		{
			_corrected = _testData.Trim();
			if (_corrected.Length==0)
			{
				_errorMessage = "WarningEmailFieldBlank";
				return false;
			}
			if (!Regex.IsMatch(_corrected,@"^([A-Za-z_0-9!#\$%\^\{\}`~&'\+\-=_.])+@([A-Za-z_0-9-.])+$"))
			{
				_errorMessage = "WarningCorrectEmail";
				return false;
			}
			return true;
		}
		/// <summary>
		/// Check for valid login. Additional info required: advanced login flag as string: "0" || "1"
		/// </summary>
		/// <returns>True, if valid. False, if invalid.</returns>
		private static bool loginCheck()
		{
			_corrected = _testData.Trim();
			if ((_corrected.Length==0)&&((_additionalInfo as string)=="1"))
			{
				_errorMessage = "WarningLoginFieldBlank";
				return false;
			}
			return true;
		}
		/// <summary>
		/// Check for valid password. Additional info required:
		/// </summary>
		/// <returns></returns>
		private static bool passwordCheck()
		{
			if (_testData.Length==0)
			{
				_errorMessage = "WarningPassBlank";
				return false;
			}
			_corrected = _testData;
			return true;
		}
		/// <summary>
		/// Check for vadid incoming mail server. Additional info required:
		/// </summary>
		/// <returns></returns>
		private static bool inServerCheck()
		{
			_corrected = _testData.Trim();
			if ((_corrected.Length == 0)&&(_additionalInfo as string)=="1")
			{
				_errorMessage = "WarningIncServerBlank";
				return false;
			}
			if (!Regex.IsMatch(_corrected,@"^([A-Za-z0-9-.])+$"))
			{
				_errorMessage = "WarningCorrectIncServer";
				return false;
			}
			return true;
		}
		/// <summary>
		/// Check for valid port number. Additional info required:
		/// </summary>
		/// <returns></returns>
		private static bool inPortCheck()
		{
			_corrected = _testData.Trim();
			if ((_corrected.Length == 0)&&((_additionalInfo as string)=="1"))
			{
				_errorMessage = "WarningIncPortBlank";
				return false;
			}
			int iPort;
			try
			{
				iPort = int.Parse(_corrected);
			}
			catch
			{
				_errorMessage = "WarningIncPortNumber";
				return false;
			}
			if ((iPort<0)||(iPort>65535))
			{
				_errorMessage = "WarningIncPortNumber";
				return false;
			}
			return true;
		}
		/// <summary>
		/// Check for valid SMTP Server string. Additional info required:
		/// </summary>
		/// <returns></returns>
		private static bool outServerCheck()
		{
			_corrected = _testData.Trim();
			if ((_corrected.Length == 0)&&((_additionalInfo as string)=="1"))
			{
				_errorMessage = "WarningOutPortBlank";
				return false;
			}
			if (!Regex.IsMatch(_corrected,@"^([A-Za-z0-9-.])+$"))
			{
				_errorMessage = "WarningCorrectSMTPServer";
				return false;
			}
			return true;
		}
		/// <summary>
		/// Check for valid out port. Additional info required:
		/// </summary>
		/// <returns></returns>
		private static bool outPortCheck()
		{
			_corrected = _testData.Trim();
			if ((_corrected.Length == 0)&&((_additionalInfo as string)=="1"))
			{
				_errorMessage = "WarningOutPortBlank";
				return false;
			}
			int iPort;
			try
			{
				iPort = int.Parse(_corrected);
			}
			catch
			{
				_errorMessage = "WarningOutPortNumber";
				return false;
			}
			if ((iPort<0)||(iPort>65535))
			{
				_errorMessage = "WarningOutPortNumber";
				return false;
			}
			return true;
		}
		/// <summary>
		/// Check for valid message per page string. Additional info required:
		/// </summary>
		/// <returns></returns>
		private static bool mppCheck()
		{
			_corrected = _testData.Trim();
			if (_corrected.Length == 0)
			{
				_errorMessage = "<undefined>";
				return false;
			}
			short mPerPage;
			try
			{
				mPerPage = short.Parse(_corrected);
			}
			catch
			{
				_errorMessage = "<undefined>";
				return false;
			}
			if (mPerPage<0)
			{
				_errorMessage = "<undefined>";
				return false;
			}
			return true;
		}
		/// <summary>
		/// Check for valid advanced ???.
		/// </summary>
		/// <returns></returns>
		private static bool advancedCheck()
		{
			_corrected = _testData.Trim();
			if (_corrected.Length == 0)
			{
				_errorMessage = "<undefined>";
				return false;
			}
			return true;
		}
		/// <summary>
		/// Check for valid smtpLogin. Additional info not required.
		/// </summary>
		/// <returns></returns>
		private static bool smtpLogin()
		{
			_corrected = _testData.Trim();
			if (_corrected.Length == 0)
			{
				_errorMessage = "<undefined>";
				return false;
			}
			return true;
		}
		/// <summary>
		/// Check for valid "keep messages on server for days" value. Additional info not required.
		/// </summary>
		/// <returns></returns>
		private static bool keepMessagesCheck()
		{
			_corrected = _testData.Trim();
			if (_corrected.Length == 0)
			{
				_errorMessage = "<undefined>";
				return false;
			}
			if (!Regex.IsMatch(_corrected,@"^([\d])+$"))
			{
				_errorMessage = "<undefined>";
				return false;
			}
			return true;
		}
		/// <summary>
		/// Check for valid "Substring" value in filters. Additional info not required.
		/// </summary>
		/// <returns></returns>
		private static bool substringCheck()
		{
			_corrected = _testData.Trim();
			if (_corrected.Length == 0)
			{
				_errorMessage = "WarningEmptyFilter";
				return false;
			}
			return true;
		}
		/// <summary>
		/// Check for valid folder name. Additional info not required.
		/// </summary>
		/// <returns></returns>
		private static bool folderNameCheck()
		{
			_corrected = _testData.Trim();
			if (_corrected.Length == 0)
			{
				_errorMessage = "WarningEmptyFolderName";
				return false;
			}
			if (Regex.IsMatch(_corrected,@"[""<>/\\\*\?\|:]+"))
			{
				_errorMessage = "WarningCorrectFolderName";
				return false;
			}
			if (Regex.IsMatch(_corrected,@"^(|CON|AUX|COM1|COM2|COM3|COM4|LPT1|LPT2|LPT3|PRN|NUL)+$",RegexOptions.IgnoreCase))
			{
				_errorMessage = "WarningCorrectFolderName";
				return false;
			}
			return true;
		}
	}
	//*****************************************************************

	/// <summary>
	/// Summary description for Utils.
	/// </summary>
	public class Utils
	{
		private Utils() { }

		public static string GetDateWithOffsetFormatString(Account acct, DateTime date)
		{
			WebmailSettings settings = (new WebMailSettingsCreator()).CreateWebMailSettings();
			int timeOffset = settings.DefaultTimeZone;
			DateTime dt = Utils.GetMessageDateWithOffset(date, GetMinutesTimesOffset(timeOffset));
			return FormatDate(acct, dt);
		}

		public static DateTime GetMessageDateWithOffset(DateTime msgDateReceivedOriginal, int timeOffsetInMinutes)
		{
			DateTime msgDate;

			msgDate = msgDateReceivedOriginal;
			if (timeOffsetInMinutes != 0 && msgDate != Constants.MinDate)
			{
				TimeZone localZone = TimeZone.CurrentTimeZone;
				if (localZone.IsDaylightSavingTime(DateTime.Now))
				{
					msgDate = msgDate.AddMinutes(timeOffsetInMinutes + 60);
				}
				else
				{
					msgDate = msgDate.AddMinutes(timeOffsetInMinutes);
				}
			} 
			return msgDate;
		}

		public static int GetMinutesTimesOffset(int timeZone)
		{
			int timeOffset = 0;
			switch (timeZone)
			{
				case 0:
					TimeZone tz = TimeZone.CurrentTimeZone;
					TimeSpan ts = tz.GetUtcOffset(DateTime.Now);
					timeOffset = ((tz.IsDaylightSavingTime(DateTime.Now)) ? ts.Hours - 1 : ts.Hours) * 60;
					timeOffset += ts.Minutes;
					break;
				case 1:
					timeOffset = -12 * 60;
					break;
				case 2:
					timeOffset = -11 * 60;
					break;
				case 3:
					timeOffset = -10 * 60;
					break;
				case 4:
					timeOffset = -9 * 60;
					break;
				case 5:
					timeOffset = -8 * 60;
					break;
				case 6:
				case 7:
					timeOffset = -7 * 60;
					break;
				case 8:
				case 9:
				case 10:
				case 11:
					timeOffset = -6 * 60;
					break;
				case 12:
				case 13:
				case 14:
					timeOffset = -5 * 60;
					break;
				case 15:
				case 16:
				case 17:
					timeOffset = -4 * 60;
					break;
				case 18:
					timeOffset = (int)(-3.5 * 60);
					break;
				case 19:
				case 20:
				case 21:
					timeOffset = -3 * 60;
					break;
				case 22:
					timeOffset = -2 * 60;
					break;
				case 23:
				case 24:
					timeOffset = -60;
					break;
				case 25:
				case 26:
					timeOffset = 0;
					break;
				case 27:
				case 28:
				case 29:
				case 30:
				case 31:
					timeOffset = 60;
					break;
				case 32:
				case 33:
				case 34:
				case 35:
				case 36:
				case 37:
					timeOffset = 2 * 60;
					break;
				case 38:
				case 39:
				case 40:
				case 41:
					timeOffset = 3 * 60;
					break;
				case 42:
					timeOffset = (int)(3.5 * 60);
					break;
				case 43:
				case 44:
					timeOffset = 4 * 60;
					break;
				case 45:
					timeOffset = (int)(4.5 * 60);
					break;
				case 46:
				case 47:
					timeOffset = 5 * 60;
					break;
				case 48:
					timeOffset = (int)(5.5 * 60);
					break;
				case 49:
					timeOffset = 5 * 60+45;
					break;
				case 50:
				case 51:
				case 52:
					timeOffset = 6 * 60;
					break;
				case 53:
					timeOffset = (int)(6.5 * 60);
					break;
				case 54:
				case 55:
					timeOffset = 7 * 60;
					break;
				case 56:
				case 57:
				case 58:
				case 59:
				case 60:
					timeOffset = 8 * 60;
					break;
				case 61:
				case 62:
				case 63:
					timeOffset = 9 * 60;
					break;
				case 64:
				case 65:
					timeOffset = (int)(9.5 * 60);
					break;
				case 66:
				case 67:
				case 68:
				case 69:
				case 70:
					timeOffset = 10 * 60;
					break;
				case 71:
					timeOffset = 11 * 60;
					break;
				case 72:
				case 73:
					timeOffset = 12 * 60;
					break;
				case 74:
					timeOffset = 13 * 60;
					break;
			}
			return timeOffset;
		}

		public static string FormatDate(Account acct, DateTime date)
		{
			return date.ToString("MM/dd/yyyy HH:mm:ss", CultureInfo.InvariantCulture);
		}

		public static string[] GetSupportedSkins(string skinsFolder)
		{
			ArrayList skinsArr = new ArrayList();
			if (Directory.Exists(skinsFolder))
			{
				string[] skins = Directory.GetDirectories(skinsFolder);
				foreach (string skin in skins)
				{
					skinsArr.Add(Path.GetFileName(skin));
				}
			}
			return (string[])skinsArr.ToArray(typeof (string));
		}

		public static int GetCurrentSkinIndex(string[] supportedSkins, string skinName)
		{
			for (int i = 0; i < supportedSkins.Length; i++)
			{
				if (string.Compare(skinName, supportedSkins[i], true, CultureInfo.InvariantCulture) == 0)
				{
					return i;
				}
			}
			return -1;
		}

		public static string[] GetSupportedLangs()
		{
			ArrayList arr = new ArrayList();
			string langsXml = Path.Combine(GetDataFolderPath(), @"langs\langs.xml");
			if (File.Exists(langsXml))
			{
				XPathDocument xpathDoc = new XPathDocument(langsXml);
				XPathNavigator nav = xpathDoc.CreateNavigator();
				XPathNodeIterator langIter = nav.Select(string.Format("langs/lang/FriendlyName"));
				while (langIter.MoveNext())
				{
					arr.Add(langIter.Current.Value);
				}
			}
			return (arr.Count > 0) ? (string[])arr.ToArray(typeof(string)) : new string[] { Constants.defaultLang };
		}

		public static string EncodeHtml(string s)
		{
			Regex rChar;
			rChar = new Regex("[\x0-\x8\xB-\xC\xE-\x1F]+");
			s = rChar.Replace(s, " ");
			s = s.Replace("&", "&amp;");
			s = s.Replace("<", "&lt;");
			s = s.Replace(">", "&gt;");
			return s;
		}

		public static string DecodeHtml(string s)
		{
			s = s.Replace("&lt;", "<");
			s = s.Replace("&gt;", ">");
			s = s.Replace("&amp;", "&");
			return s;
		}

		public static string AttributeQuote(string s)
		{
			s = s.Replace("\"", "&quot;");
			return s;
		}

		public static string EncodeCDATABody(string s)
		{
			Regex rChar;
			rChar = new Regex("[\x0-\x8\xB-\xC\xE-\x1F]+");
			s = rChar.Replace(s, " ");

			return s.Replace(@"]]>", @"&#93;&#93;&gt;");
		}

		public static string DecodeHtmlBody(string s)
		{
			return s.Replace(@"&#93;&#93;&gt;", @"]]>");
		}

		public static string EncodeJsSaveString(string s)
		{
			StringBuilder sb = new StringBuilder(s);
			sb.Replace("\\", @"\\");
			sb.Replace("'", @"\'");
			sb.Replace("\"", @"\""");
			sb.Replace("\t", @"\t");
			sb.Replace("\r", @"\r");
			sb.Replace("\n", @"\n");
			return sb.ToString();
		}

		public static string GetMD5DigestHexString(string str)
		{
			byte[] bytes = Encoding.Default.GetBytes(str);
			MD5 md5 = new MD5CryptoServiceProvider();
			bytes = md5.ComputeHash(bytes);
			return BytesToHexString(bytes);
		}

		public static string BytesToHexString(byte[] bytes)
		{
			StringBuilder result = new StringBuilder();
			foreach(byte b in bytes)
			{
				result.Append(b.ToString("x2"));
			}
			return result.ToString();
		}

		public static string GetAttachmentMimeTypeFromFileExtension(string fileExtension)
		{
			string result = "application/octet-stream";

			if ((fileExtension == null) || (fileExtension.Length == 0))
			{
				return result;
			}

			switch (fileExtension.ToLower(CultureInfo.InvariantCulture))
			{
				case "gif": return "image/gif";
				case "png": return "image/png";
				case "jpe":
				case "jpg":
				case "jpeg": return "image/jpeg";
				case "tif":
				case "tiff": return "image/tiff";
				case "bin":
				case "dms":
				case "lha":
				case "lzh":
				case "exe":
				case "class":
				case "dll": return "application/octet-stream";
				case "js": return "application/x-javascript";
				case "swf": return "application/x-shockwave-flash";
				case "doc": return "application/msword";
				case "zip": return "application/zip";
				case "ai":
				case "eps":
				case "ps": return "application/postscript";
				case "pdf": return "application/pdf";
				case "rtf": return "application/rtf";
				case "ppt": return "application/vnd.ms-powerpoint";
				case "htm":
				case "html": return "text/html";
				case "css": return "text/css";
				case "rtx": return "text/richtext";
				case "txt":
				case "asc": return "text/plain";
				case "xml": return "text/xml";
				case "wav": return "audio/x-wav";
				case "mid":
				case "midi": return "audio/midi";
				case "mpga":
				case "mp2":
				case "mp3": return "audio/mpeg";
				case "aif":
				case "aiff": return "audio/x-aiff";
				case "ra": return "audio/x-realaudio";
				case "mpeg":
				case "mpg":
				case "mpe": return "video/mpeg";
				case "qt":
				case "mov": return "video/quicktime";
				case "avi": return "video/x-msvideo";
			}


			RegistryKey regKey = null;
			try
			{
				if (fileExtension[0] == '.')
				{
					fileExtension = fileExtension.Remove(0, 1);
				}

				regKey = Registry.ClassesRoot;
				if (regKey != null)
				{
					regKey = regKey.OpenSubKey(string.Format(CultureInfo.InvariantCulture, ".{0}", fileExtension));
				}
				if (regKey != null)
				{
					result = (string)regKey.GetValue("Content Type");
				}
			}
			catch (ArgumentException)
			{
				return result;
			}
			finally
			{
				if (regKey != null) regKey.Close();
			}
			return (result == null) ? "application/octet-stream" : result;
		}

		public static string GetTempFolderName(HttpSessionState s)
		{
			string tempFolder = string.Empty;
			if (s != null)
			{
				Account acct = s[Constants.sessionAccount] as Account;
				if (acct != null)
				{
					tempFolder = Utils.GetMD5DigestHexString(s.SessionID);
					FileSystem fs = new FileSystem(acct.Email, 0, false);
					tempFolder = fs.CreateFolder(tempFolder);
				}
				if (s[Constants.sessionTempFolder] != null)
				{
					if (string.Compare(s[Constants.sessionTempFolder].ToString(), tempFolder, true, CultureInfo.InvariantCulture) != 0)
					{
						if (Directory.Exists(s[Constants.sessionTempFolder].ToString()))
						{
							Directory.Delete(s[Constants.sessionTempFolder].ToString(), true);
						}
					}
					s[Constants.sessionTempFolder] = tempFolder;
				}
				else
				{
					s.Add(Constants.sessionTempFolder, tempFolder);
				}
			}
			return tempFolder;
		}

		public static string GetTempFolderNameFromAcct(string email)
		{
			string tempFolder = string.Empty;
			if (email != string.Empty)
			{
				FileSystem fs = new FileSystem(email, 0, false);
				tempFolder = fs.CreateFolder("");
			}
			return tempFolder;
		}

		public static string ConvertToUtf7Modified(string src)
		{
			return ImapUtils.ToUtf7String(src);
		}

		public static string ConvertFromUtf7Modified(string src)
		{
			return ImapUtils.FromUtf7String(src);
		}

		public static string CreateTempFilePath(string tempFolderName, string filename)
		{
			string tempFilePath = string.Format("{0}{1}", Utils.GetMD5DigestHexString(filename), Path.GetExtension(filename));
			tempFilePath = Path.Combine(tempFolderName, tempFilePath);
			if (File.Exists(tempFilePath))
			{
				int i = 1;
				while (File.Exists(tempFilePath))
				{
					tempFilePath = string.Format("{0}_{2}{1}", Utils.GetMD5DigestHexString(filename), Path.GetExtension(filename), i);
					tempFilePath = Path.Combine(tempFolderName, tempFilePath);
					i++;
				}
			}
			if (tempFilePath.Length > Constants.PathMaxLength)
			{
				System.Diagnostics.Debug.Assert(false);
			}
			return tempFilePath;
		}

		public static string EncodePassword(string password)
		{
			if (password == null) return password;

			StringBuilder sb = new StringBuilder();
			byte[] plainBytes = Encoding.UTF8.GetBytes(password);
			if (plainBytes.Length > 0)
			{
				byte encodeByte = plainBytes[0];
				sb.Append(encodeByte.ToString("x2"));
				for (int i = 1; i < plainBytes.Length; i++)
				{
					plainBytes[i] = (byte)(plainBytes[i] ^ encodeByte);
					sb.Append(plainBytes[i].ToString("x2"));
				}
			}
			return sb.ToString();
		}

		public static string DecodePassword(string password)
		{
			string result = string.Empty;
			if ((password.Length > 0) && (password.Length % 2 == 0))
			{
				try
				{
					byte decodeByte = byte.Parse(password.Substring(0, 2), NumberStyles.HexNumber, CultureInfo.InvariantCulture);
					byte[] plainBytes = new byte[password.Length / 2];
					plainBytes[0] = decodeByte;
					if (password.Length > 2)
					{
						int startIndex = 2;
						int currentByte = 1;
						do
						{
							string hexByte = password.Substring(startIndex, 2);
							plainBytes[currentByte] = (byte)(byte.Parse(hexByte, NumberStyles.HexNumber, CultureInfo.InvariantCulture) ^ decodeByte);
							startIndex += 2;
							currentByte++;
						}
						while (startIndex < password.Length);
					}
					result = Encoding.UTF8.GetString(plainBytes);
				}
				catch
				{
					// can't parse hexByte
				}
			}
			return result;
		}

		public static string GetLogFilePath()
		{
			string logPath = string.Empty;
			string dataFolder = Utils.GetDataFolderPath();
			if (dataFolder != null)
			{
				logPath = Path.Combine(dataFolder, Constants.logFilename);
			}
			return logPath;
		}

		public static string LoadFromFile(string filename, Encoding enc, int index, int count)
		{
			FileInfo fi = new FileInfo(filename);
			if (fi.Exists)
			{
				if (count < 0) count = (int)fi.Length;
				if ((index + count) <= fi.Length)
				{
					char[] buffer = new char[count];
					using (StreamReader sr = new StreamReader(filename, enc, true))
					{
						int reads = index;
						while (count > 0)
						{
							reads += sr.ReadBlock(buffer, reads, count);
							count -= reads;
						}
					}
					return new string(buffer);
				}
			}
			return null;
		}

		public static string LoadFromFile(string filename, Encoding enc)
		{
			return LoadFromFile(filename, enc, 0, -1);
		}

		public static void ClearLog()
		{
			string logFile = GetLogFilePath();
			FileInfo fi = new FileInfo(logFile);
			if (fi.Exists)
			{
				try
				{
					using (FileStream fs = fi.Open(FileMode.Create)){}
				}
				catch {}
			}
		}

		public static Encoding GetEncodingByCodePage(int codePage)
		{
			Encoding enc = Encoding.Default;
			try
			{
				enc = Encoding.GetEncoding(codePage);
			}
			catch {}
			return enc;
		}

		public static void ReverseCollection(ref CollectionBase collection)
		{
			Stack s = new Stack(collection);
			object[] objArr = s.ToArray();

			collection.Clear();
			foreach (object obj in objArr)
			{
				((IList)collection).Add(obj);
			}
		}

		public static int RandMsgID(int id_msg)
		{
			Random rnd = new Random();
			return id_msg + rnd.Next(1, 10);
		}

		public static string GetMessagePlainReplyToBody(Account acct, MailBee.Mime.MailMessage mailMessage)
		{
			string signature = string.Empty;

			string plainBody = null;
			if ((mailMessage.BodyPlainText == null) || (mailMessage.BodyPlainText.Length == 0))
			{
				// clone because we need Attach.SavedAs
				MailBee.Mime.MailMessage cloneMsg = mailMessage.Clone();
				cloneMsg.Parser.HtmlToPlainMode = MailBee.Mime.HtmlToPlainAutoConvert.IfHtml;
				cloneMsg.Parser.HtmlToPlainOptions = MailBee.Mime.HtmlToPlainConvertOptions.AddImgAltText | MailBee.Mime.HtmlToPlainConvertOptions.AddUriForAHRef;
				cloneMsg.Parser.Apply();
				plainBody = cloneMsg.BodyPlainText;
			}
			else
			{
				plainBody = mailMessage.BodyPlainText;
			}

			string mess = "\r\n---- " + "OriginalMessage" + " ----\n";
			mess += "From" + ": " + Utils.EncodeHtml(mailMessage.From.ToString()) + "\n";
			mess += "To" + ": " + Utils.EncodeHtml(mailMessage.To.ToString()) + "\n";
			if (mailMessage.Cc.ToString().Length > 0)
				mess += "CC" + ": " + Utils.EncodeHtml(mailMessage.Cc.ToString()) + "\n";
			mess += "Sent" + ": " + Utils.DecodeHtml(Utils.GetDateWithOffsetFormatString(acct, mailMessage.Date)) + "\n";
			mess += "Subject" + ": " + Utils.EncodeHtml(mailMessage.Subject) + "\n\n";
			mess += Utils.DecodeHtml(plainBody);
			mess = mess.Replace("\n", "\n>");
			mess = "\r\n\r\n\r\n" + signature + mess;

			return mess;
		}

		public static string GetMessageHtmlReplyToBody(Account acct, MailBee.Mime.MailMessage mailMessage)
		{
			string htmlBody = null;
			if ((mailMessage.BodyHtmlText == null) || (mailMessage.BodyHtmlText.Length == 0))
			{
				htmlBody = mailMessage.BodyPlainText; // body plain already modified to HTML
			}
			else
			{
				htmlBody = mailMessage.BodyHtmlText;
			}

			string mess = @"<br/><blockquote style=""border-left: solid 2px #000000; margin-left: 5px; padding-left: 5px;"">";
			mess += "---- " + "OriginalMessage" + " ----<br/>";
			mess += "<b>" + "From" + "</b>: " + Utils.EncodeHtml(mailMessage.From.ToString()) + "<br/>";
			mess += "<b>" + "To" + "</b>: " + Utils.EncodeHtml(mailMessage.To.ToString()) + "<br/>";
			if (mailMessage.Cc.ToString().Length > 0)
				mess += "<b>" + "CC" + "</b>: " + Utils.EncodeHtml(mailMessage.Cc.ToString()) + "<br/>";
			mess += "<b>" + "Sent" + "</b>: " + Utils.EncodeHtml(Utils.GetDateWithOffsetFormatString(acct, mailMessage.Date)) + "<br/>";
			mess += "<b>" + "Subject" + "</b>: " + Utils.EncodeHtml(mailMessage.Subject) + "<br/><br/>";
			mess += htmlBody + "</blockquote>";

			return mess;
		}

		public static string GetAttachmentDownloadLink(MailBee.Mime.Attachment attach, bool isViewLink)
		{
			string attachmentName = (attach.Filename.Length > 0) ? attach.Filename : attach.Name;

			string downloadStr = "download_view_attachment.aspx?download={0}&filename={1}&temp_filename={2}";
			string filename = Path.GetFileName(attach.SavedAs);
			downloadStr = string.Format(downloadStr, (isViewLink) ? "0" : "1", HttpUtility.UrlEncode(attachmentName), HttpUtility.UrlEncode(filename));
			
			return downloadStr;
		}

        public static string GetMessageDownloadLink(string uid)
        {
            return string.Format(@"download_view_attachment.aspx?uid={0}",
                HttpUtility.UrlEncode((uid != null) ? uid : string.Empty));
        }
        
        public static string GetMessagePrintLink(MailMessage msg)
		{
			string uid = (msg is XMailMessage) ? ((XMailMessage)msg).UidXMail : msg.UidOnServer as string;
			string charset = msg.Charset as string;
			if ((msg != null))
			{
				return string.Format(@"msgprint.aspx?uid={0}",
					HttpUtility.UrlEncode((uid != null) ? uid : string.Empty));
			}
			return string.Empty;
		}

		public static string GetAttachmentIconLink(MailBee.Mime.Attachment attach)
		{
			string iconLink = string.Empty;
			switch (attach.ContentType.ToLower(CultureInfo.InvariantCulture))
			{
				case "application/asp":
					iconLink = "images/icons/application_asp.gif";
					break;
				case "application/css":
					iconLink = "images/icons/application_css.gif";
					break;
				case "application/doc":
					iconLink = "images/icons/application_doc.gif";
					break;
				case "application/html":
					iconLink = "images/icons/application_html.gif";
					break;
				case "application/pdf":
					iconLink = "images/icons/application_pdf.gif";
					break;
				case "application/xls":
					iconLink = "images/icons/application_xls.gif";
					break;
				case "image/bmp":
					iconLink = "images/icons/image_bmp.gif";
					break;
				case "image/gif":
					iconLink = "images/icons/image_gif.gif";
					break;
				case "image/jpg":
				case "image/jpeg":
					iconLink = "images/icons/image_jpeg.gif";
					break;
				case "image/tif":
				case "image/tiff":
					iconLink = "images/icons/image_tiff.gif";
					break;
				case "text/plain":
					iconLink = "images/icons/text_plain.gif";
					break;
				default:
					iconLink = "images/icons/attach.gif";
					break;
			}
			return iconLink;
		}

		public static string GetFriendlySize(long byteSize)
		{
			WebmailResourceManager man = (new WebmailResourceManagerCreator()).CreateResourceManager();
			double size = Math.Round((double)byteSize / 1024);
			double mbSize = size / 1024;
			string str = "{0}{1}";
			return (mbSize > 1) ? string.Format(str, Math.Ceiling(mbSize * 10)/10, man.GetString("Mb")) : string.Format(str, size, man.GetString("Kb"));
		}

		public static string GetShortFilename(string filename)
		{
			if (filename.Length > 15)
			{
				return string.Format(@"{0}...", filename.Substring(0, 12));
			}
			return filename;
		}

		public static int GetCodePageNumber(string codePageNum)
		{
			int result = -1;

			for(int i = 0; Constants.PageName.Length > i; i++)
			{
				if(Constants.PageName[i] == codePageNum)
				{
					result = Constants.PageNumber[i];
				}
			}

			return result;
		}

		public static string GetCodePageName(int codePageNum)
		{
			string result = "-1";

			for(int i = 0; Constants.PageNumber.Length > i; i++)
			{
				if(Constants.PageNumber[i] == codePageNum)
				{
					result = Constants.PageName[i];
				}
			}

			return result;
		}

		public static string ConvertHtmlToPlain(string htmlText)
		{
			MailBee.Mime.MailMessage msg = new MailMessage();
			msg.BodyHtmlText = htmlText;
			msg.MakePlainBodyFromHtmlBody();
			return msg.BodyPlainText;
		}

		public static string MakeHtmlBodyFromPlainBody(string bodyPlain)
		{
			StringBuilder sb = new StringBuilder(EncodeHtml(bodyPlain));
			string urlPattern = @"(http|https|ftp|nntp|file|telnet|gopher|wais|prospero)://[\w\d!\$&'\(\)\*\+-\./:\?@_~%=;,#]+";//!$&'()*+-./:?@_~%=;,
			Regex r = new Regex(urlPattern, RegexOptions.IgnoreCase | RegexOptions.Singleline);//-_.!~*'();/?:@&=+,$%
			Match m = r.Match(sb.ToString(), 0);
			while (m.Success)
			{
				string link = string.Format(CultureInfo.InvariantCulture, @"<a href=""{0}"">{0}</a>", m.Value);

				sb.Replace(m.Value, link, m.Index, m.Length);
				m = r.Match(sb.ToString(), m.Index + link.Length);
			}

			string quotedBody = "";
			string[] splittedBody = sb.ToString().Split(new Char[] { '\n' });
			for (int i = 0; i < splittedBody.Length; i++)
			{
				Regex lineRegex = new Regex("(\r){1}", RegexOptions.Singleline);
				string bodyLine = lineRegex.Replace(splittedBody[i], "");

				lineRegex = new Regex("^.{0,19}&gt;", RegexOptions.Multiline);
				if (lineRegex.IsMatch(bodyLine))
				{
					quotedBody += @"<font class=""wm_message_body_quotation"">" + bodyLine + "</font><br />";
				}
				else
				{
					quotedBody += bodyLine + "<br />";
				}
			}

            // MailBee.Html
            Processor pr = new Processor();
			pr.Dom.OuterHtml = quotedBody;
            RuleSet rs = new RuleSet();

            TagAttributeCollection attrsToAdd = new TagAttributeCollection();
            MailBee.Html.TagAttribute addAttr = new MailBee.Html.TagAttribute();
            addAttr.Name = "target";
            addAttr.Value = "\"_blank\"";
            attrsToAdd.Add(addAttr);
            rs.AddTagProcessingRule("a", null, attrsToAdd, null, false);

            pr.Dom.Process(rs, null);

            return pr.Dom.OuterHtml;
        }

		public static ArrayList Split(string src, string separator)
		{
			ArrayList result = new ArrayList();
			int startIndex = 0;
			while (startIndex < src.Length)
			{
				int index = src.IndexOf(separator, startIndex);
				if (index >= 0)
				{
					result.Add(src.Substring(startIndex, index - startIndex));
					startIndex = index + separator.Length;
				}
				else
				{
					result.Add(src.Substring(startIndex, src.Length - startIndex));
					break;
				}
			}
			return result;
		}

		public static string GetDelimeter()
		{
			string delimeter = "";

			if(Environment.Version.Major == 1)
			{
				delimeter = ":";
			}
			else
			{
				delimeter = "$";
			}

			return delimeter;
		}

		public static string GetAddressesFriendlyName(EmailAddressCollection coll)
		{
			if (coll == null) return string.Empty;

			StringBuilder sb = new StringBuilder();
			foreach (EmailAddress addr in coll)
			{
				sb.AppendFormat(@"{0}, ",
					(addr.DisplayName != null && addr.DisplayName.Length > 0) ? addr.DisplayName : addr.Email);
			}
			if (sb.Length > 2) sb.Remove(sb.Length - 2, 2);
			return sb.ToString();
		}

		public static string GetDataFolderPath()
		{
			string strDataFolderPath = string.Empty;
			if (HttpContext.Current.Application[Constants.appSettingsDataFolderPath] != null)
			{
				strDataFolderPath = (string)HttpContext.Current.Application[Constants.appSettingsDataFolderPath];
			}
			strDataFolderPath = ConfigurationManager.AppSettings[Constants.appSettingsDataFolderPath];

			/*converting relative path in absolute*/
			Uri appUri = new Uri(HttpContext.Current.Request.PhysicalApplicationPath);
			Uri newUri = new Uri(appUri, strDataFolderPath);
			strDataFolderPath = newUri.LocalPath;

			return strDataFolderPath;
		}

		public static string RemoveATag(string source)
		{
			string tmp_src=source.ToLower();
			int index=tmp_src.IndexOf("<a ");
			int last_index=index+3;
			while(index >= 0)
			{
				int sub_ind=tmp_src.IndexOf(">",index)+1;
				int sub_ind_last=tmp_src.IndexOf("</a>",sub_ind);
				last_index=sub_ind_last+4;

				string tmp_url=source.Substring(sub_ind,sub_ind_last-sub_ind);

				tmp_src=tmp_src.Remove(index,last_index-index);
				tmp_src=tmp_src.Insert(index,tmp_url);

				source=source.Remove(index,last_index-index);
				source=source.Insert(index,tmp_url);

				if(last_index>=tmp_src.Length)
				{
					index=-1;
					continue;
				}
				index=tmp_src.IndexOf("<a ",last_index);
			}
			return source;
		}

		public static Hashtable ReadWebmailTab(string xmailRootPath)
		{
			if (xmailRootPath == null) return null;
			Hashtable result = null;
			if (Directory.Exists(xmailRootPath))
			{
				string path = Path.Combine(xmailRootPath, Constants.webmailTab);
				if (File.Exists(path))
				{
					result = new Hashtable();
					try
					{
						using (StreamReader sr = new StreamReader(File.OpenRead(path)))
						{
							string str = null;
							while ((str = sr.ReadLine()) != null)
							{
								str = str.Trim();
								if (str.StartsWith("#")) continue;
								string[] keyValue = str.Split(new char[] { '\t' });
								if (keyValue.Length == 2)
								{
									string key = keyValue[0].ToString().Trim(new char[] { '"' });
									string value = keyValue[1].ToString().Trim(new char[] { '"' });
									if (!result.ContainsKey(key))
									{
										result.Add(key, value);
									}
								}
							}
						}
					}
					catch (Exception ex)
					{
						throw new WebMailIOException(ex);
					}
				}
			}
			return result;
		}
	}
}
